#include "libgraph.h"



static void gr_font(fontname)
     char * fontname;
{
  XFontStruct * font;

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_font: load_font=%s\n",fontname);
#endif 
  font = XLoadQueryFont(grdisplay, fontname);
  if (font == NULL) gr_fail("cannot find font %s", fontname);

  if (grwindows[current_window].grfont != NULL) 
     XFreeFont(grdisplay, grwindows[current_window].grfont);

  grwindows[current_window].grfont = font;

  if(ValidWindow!=FALSE) 
    XSetFont(grdisplay, grwindows[current_window].gc, 
             grwindows[current_window].grfont->fid);
#ifdef DEBUG_TEXT 
   fprintf(stderr,"gr_font: end\n");
#endif 
  XFlush(grdisplay);
}



value gr_set_font(font_type, font_att, font_size)
     value font_type, font_att, font_size;
{
  char Font_name[256];
  char Font_type[32];
  char Font_att[32];
  char Font_size[32];
  char Slant[8];

#ifdef DEBUG_TEXT 
   fprintf(stderr,"gr_set_font: begin\n");
#endif 
  gr_check_open();

  if(ValidWindow==FALSE) return Val_unit;

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_set_font: font_type = %d",Tag_val(font_type));
   fprintf(stderr,", font_att = %d",Tag_val(font_att));
   fprintf(stderr,", font_size = %d\n",Tag_val(font_size));
#endif 


  switch(Tag_val(font_att))
    {
    case 0: /* Normal */
      strcpy(Font_att,"medium");
      strcpy(Slant,"r");
      grwindows[current_window].underline = FALSE;
      break;
    case 1: /* Italic */
      strcpy(Font_att,"medium");
      grwindows[current_window].underline = FALSE;
      strcpy(Slant,"i");
      break;
    case 2:      /* Bold */
      strcpy(Font_att,"bold");
      strcpy(Slant,"r");
      grwindows[current_window].underline = FALSE;
      break;
    case 3:      /* Underline */
      strcpy(Font_att,"medium");
      strcpy(Slant,"r");
      grwindows[current_window].underline = TRUE;
      break;
    default:  /* Bad value */
      fprintf(stderr,"set_font: wrong font attribute\n");
      strcpy(Font_att,"medium");
      strcpy(Slant,"r");
      grwindows[current_window].underline = FALSE;
      break;
    }


  switch(Tag_val(font_size))
    {
      case 0: strcpy(Font_size,"240"); break;
      case 1: strcpy(Font_size,"180"); break;
      case 2: strcpy(Font_size,"140"); break;
      case 3: strcpy(Font_size,"120"); break;
      case 4: strcpy(Font_size,"100"); break;
      case 5: strcpy(Font_size,"80"); break;
      default: fprintf(stderr,"set_font: wrong font size\n"); 
               strcpy(Font_size,"120"); break;
    }


  switch(Tag_val(font_type))
    {
      case 0: /* Default font */
              strcpy(Font_type,"courier"); 
              strcpy(Font_att,"medium");
              strcpy(Slant,"r");
              strcpy(Font_size,"120"); break;
              break;
      case 1: /* Fixed font */
              strcpy(Font_type,"courier"); 
              break;
      case 2: /* Proportional font */
              strcpy(Font_type,"charter"); 
              break;
      default: /* bad value */
              fprintf(stderr,"set_font: wrong font type\n"); 
              strcpy(Font_type,"courier"); 
              break;
    }


  strcpy(Font_name,"*");
  strcat(Font_name,Font_type);
  strcat(Font_name,"-");
  strcat(Font_name,Font_att);
  strcat(Font_name,"-");
  strcat(Font_name,Slant);
  strcat(Font_name,"-*-");
  strcat(Font_name,Font_size);
  strcat(Font_name,"-*");

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_set_font: %s\n",Font_name);
#endif 

  gr_font(Font_name);

  return Val_unit;
}

static void gr_draw_text(txt, len)
     char * txt;
     int len;
{
  int x, y, x_begin;

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_draw_text: Validwin=%d; text=%s\n",ValidWindow,txt);
#endif 

  if(ValidWindow==FALSE) return;

  x_begin = grwindows[current_window].grx;
  x = x_begin + XTextWidth(grwindows[current_window].grfont, txt, len);
  y = Wcvt(grwindows[current_window].gry) - 
      grwindows[current_window].grfont->descent + 1;

  XDrawString(grdisplay, grwindows[current_window].win, 
              grwindows[current_window].gc,
              x_begin, y, txt, len);
  if(grwindows[current_window].underline==TRUE)
      XDrawLine(grdisplay, grwindows[current_window].win, 
	        grwindows[current_window].gc, x_begin, y+1, x, y+1);
  grwindows[current_window].grx = x;
}

value gr_draw_char(chr)
     value chr;
{
  char str[1];

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_draw_char: begin\n");
#endif 

  gr_check_open();
  str[0] = Int_val(chr);
  gr_draw_text(str, 1);

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_draw_char: begin\n");
#endif 

  return Val_unit;
}
  
value gr_draw_string(str)
     value str;
{

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_draw_string: begin\n");
#endif 

  gr_check_open();
  gr_draw_text(String_val(str), string_length(str));

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_draw_string: end\n");
#endif 

  return Val_unit;
}

value gr_text_size(str)
     value str;
{
  int width;
  value res;
  char *string=String_val(str);
  int len=string_length(str);

#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_text_size: Validwin=%d; text=%s, length=%d\n",
           ValidWindow, string, len);
#endif 

  gr_check_open();

  if(len!=0)
    width = XTextWidth(grwindows[current_window].grfont, 
                       String_val(str), string_length(str));
  else width=0;

  res = alloc_tuple(2);
  Field(res, 0) = Val_int(width);
  Field(res, 1) = Val_int(grwindows[current_window].grfont->ascent + 
                          grwindows[current_window].grfont->descent);
#ifdef DEBUG_TEXT
   fprintf(stderr,"gr_text_size: return x=%d\n",width);
#endif 

  return res;
}
